# -*- coding: utf-8 -*-

# Copyright (C) 2024 - 2026 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides the Scade predefined operator codes as Python constants."""

SC_ECK_NONE = 1
SC_ECK_AND = 2
SC_ECK_OR = 3
SC_ECK_XOR = 4
SC_ECK_NOT = 5
SC_ECK_SHARP = 6
SC_ECK_PLUS = 7
SC_ECK_SUB = 8
SC_ECK_NEG = 9
SC_ECK_MUL = 10
SC_ECK_REAL2INT = 11
SC_ECK_INT2REAL = 12
SC_ECK_SLASH = 14
SC_ECK_DIV = 15
SC_ECK_MOD = 16
SC_ECK_PRJ = 18
SC_ECK_CHANGE_ITH = 19
SC_ECK_LESS = 20
SC_ECK_LEQUAL = 21
SC_ECK_GREAT = 22
SC_ECK_GEQUAL = 23
SC_ECK_EQUAL = 24
SC_ECK_NEQUAL = 25
SC_ECK_PRE = 26
SC_ECK_WHEN = 28
SC_ECK_FOLLOW = 29
SC_ECK_FBY = 30
SC_ECK_IF = 31
SC_ECK_CASE = 32
SC_ECK_SEQ_EXPR = 33
SC_ECK_BLD_STRUCT = 34
SC_ECK_MAP = 35
SC_ECK_FOLD = 36
SC_ECK_MAPFOLD = 37
SC_ECK_MAPI = 38
SC_ECK_FOLDI = 39
SC_ECK_SCALAR_TO_VECTOR = 40
SC_ECK_BLD_VECTOR = 41
SC_ECK_PRJ_DYN = 42
SC_ECK_MAKE = 43
SC_ECK_FLATTEN = 44
SC_ECK_MERGE = 45
SC_ECK_REVERSE = 46
SC_ECK_TRANSPOSE = 47
SC_ECK_TIMES = 49
SC_ECK_MATCH = 50
SC_ECK_SLICE = 51
SC_ECK_CONCAT = 52
SC_ECK_ACTIVATE = 53
SC_ECK_RESTART = 54
SC_ECK_FOLDW = 55
SC_ECK_FOLDWI = 56
SC_ECK_ACTIVATE_NOINIT = 57
SC_ECK_CLOCKED_ACTIVATE = 58
SC_ECK_CLOCKED_NOT = 59
SC_ECK_POS = 60
SC_ECK_MAPW = 61
SC_ECK_MAPWI = 62
SC_ECK_NUMERIC_CAST = 63
SC_ECK_MAPFOLDI = 64
SC_ECK_MAPFOLDW = 65
SC_ECK_MAPFOLDWI = 66
SC_ECK_LAND = 67
SC_ECK_LOR = 68
SC_ECK_LXOR = 69
SC_ECK_LNOT = 70
SC_ECK_LSL = 71
SC_ECK_LSR = 72
